// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 

// Copyright 2003 Liam Girdwood  

#include "asteroid.hh"
#include <libnova.h>
#include <libgnomecanvasmm/group.h>
#include <libgnomecanvasmm/ellipse.h>

namespace Castor
{
	
Asteroid::Asteroid()
{
}

Asteroid::Asteroid(Glib::ustring& name, Glib::ustring& id, struct ln_ell_orbit& orbit,
					double H, double A, double G)
{
	m_name = name;
	m_cat_no = id;
	m_H = H;
	m_G = G;
	m_A = A;
	m_orbit = orbit;
}

Asteroid::~Asteroid()
{
}

/*! \fn void Asteroid::get_equ_posn (double JD, ln_equ_posn* posn)
* \brief Get object equatorial position.
*/
void Asteroid::get_equ_posn (double JD, double& ra, double& dec)
{
	struct ln_equ_posn posn;
	get_ell_body_equ_coords (JD, &m_orbit, &posn);
	ra = posn.ra;
	dec = posn.dec;
}

/*! \fn void Asteroid::get_hrz_posn (double JD, ln_lnlat_posn* observer, ln_hrz_posn* posn)
* \brief Get object horizontal position
*/
void Asteroid::get_hrz_posn (double JD, ln_lnlat_posn* observer, ln_hrz_posn* posn)
{
	struct ln_equ_posn equ_posn;
	double ra, dec;
		
	get_equ_posn(JD, ra, dec);
	equ_posn.ra = ra;
	equ_posn.dec = dec;
	get_hrz_from_equ (&equ_posn, observer, JD, posn);
}

/*! \fn void Asteroid::get_rst_time (double JD, ln_lnlat_posn* observer, ln_rst_time* time)
* \brief Get object rise, transit and set time.
*/
void Asteroid::get_rst_time (double JD, ln_lnlat_posn* observer, ln_rst_time* time)
{
	get_ell_body_rst(JD, observer, &m_orbit, time);
}

/*! \fn double Asteroid::get_mag(double JD)
* \brief Get object Magnitude
*/
double Asteroid::get_mag(double JD)
{
	get_asteroid_mag (JD, &m_orbit, m_H, m_G);
}
		
/*! \fn void Asteroid::get_id(std::string& id)
* \brief Get object identification number.
*/
void Asteroid::get_id(std::string& id)
{
	id = m_cat_no;
}	

/*! \fn void Asteroid::get_name(std::string& name)
* \brief Get object name.
*/
void Asteroid::get_name(std::string& name)
{
	name = m_name;
}

/*! \fn void Asteroid::get_info (std::list<Glib::ustring>& value)
* \brief Get object information
*/
void Asteroid::get_info (std::list<Glib::ustring>& value)
{
}

/*! \fn void Asteroid::render(double x, double y, double mag_max, Gnome::Canvas::Group& group)
* \brief Render object
*/
void Asteroid::render(double x, double y, double mag_max, Gnome::Canvas::Group& group, bool bright, double ppd)
{
	// draw object on canvas
	double size = (mag_max + 1) - get_neptune_magnitude(m_render_JD);
	double sdiam = (get_sdiam_arc(m_render_JD) / 240.0) * ppd;
	if (size < sdiam)
		size = sdiam;
	double shadow = 1.1 * size;
	Gnome::Canvas::Ellipse *ellipse_back, *ellipse;
	
	if (!bright) {
		// draw the background shadow object
		ellipse_back = Gtk::manage (new Gnome::Canvas::Ellipse(group, x - shadow, y - shadow, x + shadow, y + shadow));
		*ellipse_back << Gnome::Canvas::Properties::fill_color("black");
		ellipse_back->show();
	}
	
	// draw real colour object
	ellipse = Gtk::manage (new Gnome::Canvas::Ellipse(group, x - size, y - size, x + size, y + size));
	ellipse->set_data (Glib::Quark("object"),(gpointer)this);
	*ellipse << Gnome::Canvas::Properties::fill_color("white");
	ellipse->show();
}

double Asteroid::get_earth_dist (double JD)
{
	return get_ell_body_earth_dist (JD, &m_orbit);
}

double Asteroid::get_sun_dist (double JD)
{
	return get_ell_body_solar_dist (JD, &m_orbit);
}

double Asteroid::get_sdiam_km ()
{
	return get_asteroid_sdiam_km(m_H, m_A);
}

double Asteroid::get_sdiam_arc (double JD)
{
	return get_asteroid_sdiam_arc(JD, &m_orbit, m_H, m_A);
}

void Asteroid::set_render_jd (double JD)
{
	m_render_JD = JD;
}

}
